/* Vampire Attack 2 Construction Set
 * Copyright (C) 2009-2010 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Billy D. Spelchan
 * @version 1.00
 */
package com.blazinggames.va2cs;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;

public class TilemapViewer 
extends Canvas 
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -725476896076268996L;

// ***** VARIABLES **************************

		protected Image _tilesetImage;
		protected Dimension _tileSize;
		/** Size to render tile on screen - this allows for scaling */
		protected Dimension _displaySize;
		protected Tilemap _tilemap;
		protected Point _position;
		protected Rectangle _tileArea;
		protected boolean _showGrid;
		
// ***** CONSTRUCTORs *****************************

		/**
		 * 
		 */
		public TilemapViewer() 
		{
			super();

			_tilesetImage = null;
			_tileSize = new Dimension(1, 1);
			_displaySize = new Dimension(1, 1);
			_tilemap = null;
			_position = new Point(0, 0);
			_tileArea = new Rectangle();
			_showGrid = false;
		}

		public TilemapViewer(Tilemap map, Image i, int tileWidth, int tileHeight)
		{
			this();
			
			_tilemap = map;
			setImage(i, tileWidth, tileHeight);
		}


// ***** METHODS - From Component *****************************************
		
		public void paint(Graphics g)
		{
			if ((_tilemap == null) || (_tilesetImage == null))
			{
				g.setColor(Color.black);
				g.fillRect(0, 0, getWidth(), getHeight());
			}
			else
			{
				int cols = Math.min(_tilemap.getWidth() - _position.x, getWidth() / _displaySize.width);
				int rows = Math.min(_tilemap.getHeight() - _position.y, getHeight() / _displaySize.height);
				int tilesperrow = Math.max(1, _tilesetImage.getWidth(this) / _tileSize.width);
				int tile, sx, sy, dx, dy;
				_tileArea.setBounds(_position.x, _position.y, cols, rows);
				g.setColor(Color.cyan);
				for (int cntrX = 0; cntrX < cols; ++cntrX)
				{
					for (int cntrY = 0; cntrY < rows; ++cntrY)
					{
						tile = _tilemap.getTile(cntrX+_position.x, cntrY+_position.y);
						dx = cntrX * _displaySize.width;
						dy = cntrY * _displaySize.height;
						sx = (tile % tilesperrow) * _tileSize.width;
						sy = (tile / tilesperrow) * _tileSize.height;
						
						g.drawImage(_tilesetImage, 
									dx, dy,
									dx+_displaySize.width, dy+_displaySize.height,
									sx, sy,
									sx + _tileSize.width, sy + _tileSize.height,
									Color.black, this);
						if (_showGrid)
							g.drawRect(dx, dy, dx+_displaySize.width, dy + _displaySize.height);
					}
				}
			}
		}

		public Dimension getPreferredSize()
		{
			if (_tilemap == null)
				return new Dimension(1, 1);
	        return new Dimension(_displaySize.width * _tilemap.getWidth(), _displaySize.height * _tilemap.getHeight());
		}

// ***** METHODS - Getting and Setting **********************************************

		public void setTilemap(Tilemap map)
		{
			_tilemap = map;
		}
		
		public void setImage(Image i, int tilewidth, int tileheight)
		{
			_tilesetImage = i;
			_tileSize.setSize(tilewidth, tileheight);
			_displaySize.setSize(tilewidth, tileheight);
		}

		public void setTileDisplaySize(int width, int height)
		{
			_displaySize.setSize(width, height);
		}
		
		public Dimension getTileDisplaySize()
		{
			return _displaySize;
		}
		
		public void setPosition(Point p)
		{
			_position.setLocation(p.x, p.y);
		}
		
		public Point getPosition()
		{
			return _position;
		}
		
		public void setGrid(boolean b)
		{
			_showGrid = b;
		}
		public boolean isGridShowing()
		{
			return _showGrid;
		}

}
