/* Vampire Attack 2 Construction Set
 * Copyright (C) 2009-2010 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Billy D. Spelchan
 * @version 1.00
 */
package com.blazinggames.va2cs;

import java.awt.AWTEvent;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.Image;
import java.awt.Label;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.Point;
import java.awt.Scrollbar;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;

public class VA2CS 
extends Frame 
implements ActionListener, AdjustmentListener, MouseListener, MouseMotionListener 
{
/**
	 * 
	 */
	private static final long serialVersionUID = 5450737810679193130L;
//	*********************
//	***** CONSTANTS *****
//	*********************

	 private static final String FILE_NEW = "New Map";
	 private static final String NEW_OUTDOOR = "Outside";
	 private static final String NEW_CASTLE = "Castle";
	 private static final String NEW_UNDERGROUND = "Underground";
	 private static final String HELP_ABOUT = "About";
	
	 private static final String TILE_RESOURCE = "va2tiles.gif";
	 private static final String TITLE_RESOURCE = "vacs.gif";
	 
	 private static String BASE64ENC = "-+abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
	 
	 public static final int MAP_SIZE = 64;
    
	 // *** VARIABLES ***
	 TilemapViewer _mapviewer;
	 GridButton _modeGrid, _tileGrid;
	 Image _tileImage, _modeImage;
	 Tilemap _map;
	 Scrollbar _vbar, _hbar;
	 Label _info;
	
	 FileDialog fdSave, fdOpen;
	 Layout _layout;
	 /**
		 The constructor class.  Adjusts the windows size and title, creates
		 the menu bar, then enables the close window event.
	 */
	 public VA2CS()
	 {
		 // Make menu bar.
		 initMenus();

		 // Set up the window
		 setTitle("Vampire Attack 2 construction Set");
		 setResizable(true);
		 initLayout();
		 // Make it possible to close the window
		 enableEvents(WindowEvent.WINDOW_CLOSING);
		 pack();
	 }

	 private void initLayout()
	 {
		 Toolkit tk = Toolkit.getDefaultToolkit();

		  URL url;
		  
		  try {
		  url = this.getClass().getResource(TILE_RESOURCE);
		  _tileImage = tk.getImage(url);
		  } catch (Exception failedLoadingTiles) {
			 _tileImage = tk.getImage(TILE_RESOURCE);
		  }
		  try {
		  url = this.getClass().getResource(TITLE_RESOURCE);
		  _modeImage = tk.getImage(url);
		  } catch (Exception failedLoadingTitle) {
			  _modeImage = tk.getImage(TITLE_RESOURCE);
		  }
		 _map = new Tilemap(64, 64);
		 _map.clear(0);
		 _map.setOutOfBoundsTile(15);
		 _modeGrid = new GridButton(_modeImage, 64, 64, 1, 1);
		 _modeGrid.setButtonsPerRow(1);
		 _tileGrid = new GridButton(_tileImage, 16, 16, 16, 2);
		 _tileGrid.setButtonsPerRow(4);
		 _mapviewer = new TilemapViewer(_map, _tileImage, 16, 16);
		 _mapviewer.addMouseListener(this);
		 _mapviewer.addMouseMotionListener(this);

		 _layout = new Layout();
		 _hbar = new Scrollbar(Scrollbar.HORIZONTAL, 0,1, 0,63);
		 _hbar.addAdjustmentListener(this);
		 _vbar = new Scrollbar(Scrollbar.VERTICAL, 0,1, 0,63);
		 _vbar.addAdjustmentListener(this);
		 _info = new Label("Info");
		 _info.setVisible(true);
		 setLayout(_layout);
		 add(Layout.VIEWER, _mapviewer);
		 add(Layout.VIEWER_HBAR, _hbar);
		 add(Layout.VIEWER_VBAR, _vbar);
		 add(Layout.MODE, _modeGrid);
		 add(Layout.TILE, _tileGrid);
		 add(Layout.INFO, _info);
	 }
    
	 /*
		 Create the menus for the window
	  */
	 private void initMenus() {
		 MenuBar mb = new MenuBar();
		 // FILE menu
		 Menu m = new Menu("File");
		 Menu mn = new Menu(FILE_NEW);
		 MenuItem mi = new MenuItem(NEW_OUTDOOR);
		 mi.addActionListener(this);
		 mn.add(mi);
		 mi = new MenuItem(NEW_CASTLE);
		 mi.addActionListener(this);
		 mn.add(mi);
		 mi = new MenuItem(NEW_UNDERGROUND);
		 mi.addActionListener(this);
		 mn.add(mi);
		 m.add(mn);
		 mi = new MenuItem("Open...");
		 mi.addActionListener(this);
		 m.add(mi);
		 mi = new MenuItem("Save");
		 mi.addActionListener(this);
		 m.add(mi);
		 mi = new MenuItem("Save As...");
		 mi.addActionListener(this);
		 m.add(mi);
		 mi = new MenuItem("-");
		 m.add(mi);
		 mi = new MenuItem("Quit");
		 mi.addActionListener(this);
		 m.add(mi);
		 mb.add( m );
/*
		 // ABOUT Menu
		 m = new Menu("Help");
		 mi = new MenuItem(HELP_ABOUT);
		 mi.addActionListener(this);
		 m.add(mi);
		 mb.add( m );
*/  
		 setMenuBar( mb );
	 }


	 private void menu_FileNew(int clearTile) {
//		 System.out.println("File->new called");
		 _map.resize(MAP_SIZE, MAP_SIZE);
		 _map.clear(clearTile);
		 Dimension ds = _mapviewer.getTileDisplaySize();
		 Dimension cs = _mapviewer.getSize();
		 _vbar.setMaximum(MAP_SIZE);
		 _hbar.setMaximum(MAP_SIZE);
		 _mapviewer.setPosition(new Point(0,0));
		 _vbar.setValue(0);
		 _hbar.setValue(0);
		 _vbar.setVisibleAmount(cs.height / Math.max(1,ds.height));
		 _hbar.setVisibleAmount(cs.width / Math.max(1,ds.width));
		 repaint();
		 _mapviewer.repaint();
		 fdSave = null;
	 }

	 private void menu_FileOpen() {
//		 System.out.println("File->Open");
		 if (fdOpen == null)
		 {
			 fdOpen = new FileDialog(this, "Open Map", FileDialog.LOAD);
		 }
		 fdOpen.setVisible(true);
		 if (fdOpen.getFile() != null) {
//			 System.out.println("load file: " + fdOpen.getDirectory() + fdOpen.getFile());
			 doLoad(fdOpen.getDirectory() + fdOpen.getFile());
		 }
	 }

	 private void menu_FileSave() {
		 if (fdSave == null) {
			 menu_FileSaveAs();
			 return;
		 }
		 if (fdSave.getFile() == null) {
			 menu_FileSaveAs();
			 return;
		 }
//		 System.out.println("Save file: " + fdSave.getDirectory() + fdSave.getFile());
		 doSave(fdSave.getDirectory() + fdSave.getFile());
	 }

	 private void menu_FileSaveAs() {
		 if (fdSave == null)
			 fdSave = new FileDialog(this, "Save map as", FileDialog.SAVE);
		 fdSave.setVisible(true);
		 if (fdSave.getFile() != null) {
//			 System.out.println("Save file as: " + fdSave.getDirectory() + fdSave.getFile());
			 doSave(fdSave.getDirectory() + fdSave.getFile());
		 }
	 }

	 private void menu_HelpAbout()
	 {
	 }
	
	 /**
		 Handle menu events (well, the quit option) and Panel actions.
	 */
	 public void actionPerformed(ActionEvent e)
	 {
		 String sID = e.getActionCommand();

		 // If selected menu is Quit
		 if (sID.equals(NEW_OUTDOOR) )
			 menu_FileNew(0);
		 else if (sID.equals(NEW_CASTLE) )
			 menu_FileNew(3);
		 else if (sID.equals(NEW_UNDERGROUND) )
			 menu_FileNew(6);
		 else if ( sID.equals( "Open..." ) )
			 menu_FileOpen();
		 else if ( sID.equals( "Save" ) )
			 menu_FileSave();
		 else if ( sID.equals( "Save As..." ) )
			 menu_FileSaveAs();
		 else if ( sID.equals( "Quit" ) )
		 {
			 setVisible(false);
			 dispose();
			 System.exit(0);
		 }
		 else if ( sID.equals(HELP_ABOUT))
			 menu_HelpAbout();
	 }


	 /**
		 Handle closing of the window
	 */
	 protected void processEvent(AWTEvent e) {
		 // See of window being closed
		 if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			 setVisible(false);
			 dispose();
			 System.exit(0);
		 }
		 else if (e.getID() == WindowEvent.COMPONENT_RESIZED)
		 {
			 Dimension ds = _mapviewer.getTileDisplaySize();
			 Dimension cs = _mapviewer.getSize();
//			 Point p = _mapviewer.getPosition();
			 _vbar.setVisibleAmount(cs.height / Math.max(1,ds.height));
			 _hbar.setVisibleAmount(cs.width / Math.max(1,ds.width));
//			 System.out.println("Window resized");
		 }
		 // if not let java handle the event
		 super.processEvent(e);
	 }


//	*********************************************
//	***** METHODS - From AdjustmentListener *****
//	*********************************************

	 public void adjustmentValueChanged(AdjustmentEvent e)
	 {
		 int topx = _hbar.getValue();
		 int topy = _vbar.getValue();
		 _mapviewer.setPosition(new Point(topx, topy));
		 _info.setText("Top X = " + topx + ", Top Y = " + topy);
//		 System.out.println(_info.getText());
		 repaint();
		 _mapviewer.repaint();
	 }
	
//	****************************************
//	***** METHODS - From MouseListener *****
//	****************************************

	 /* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
		 */
	 public void mouseClicked(MouseEvent arg0) {

	 }

	 /* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
		 */
	 public void mousePressed(MouseEvent e)
	 {
		 Point p = _mapviewer.getPosition();
		 int x = e.getX() / _mapviewer.getTileDisplaySize().width + p.x;
		 int y = e.getY() / _mapviewer.getTileDisplaySize().height + p.y;
//		 System.out.println("Mouse clicked at " + x + ", " + y);
		 if (_modeGrid.getSelected() == 0)
			 _map.setTile(x, y, _tileGrid.getSelected());
		 _mapviewer.repaint();
	 }

	 /* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
		 */
	 public void mouseReleased(MouseEvent e) 
	 {
	 }

	 /* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
		 */
	 public void mouseEntered(MouseEvent arg0) 
	 {
	 }

	 /* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
		 */
	 public void mouseExited(MouseEvent arg0) 
	 {
	 }

//	**********************************************
//	***** METHODS - From MouseMotionListener *****
//	**********************************************

	 public void mouseDragged(MouseEvent e)
	 {
		 Point p = _mapviewer.getPosition();
		 int x = e.getX() / _mapviewer.getTileDisplaySize().width + p.x;
		 int y = e.getY() / _mapviewer.getTileDisplaySize().height + p.y;
//		 System.out.println("Mouse dragged at " + x + ", " + y);
		 if (_map.getTile(x, y) != _tileGrid.getSelected())
		 {
			 if (_modeGrid.getSelected() == 0)
				 _map.setTile(x, y, _tileGrid.getSelected());
			 _mapviewer.repaint();
		 }
	 }

	 public void mouseMoved(MouseEvent e)
	 {
        
	 }

//	***** METHODS - saving/loading *****

	 public void doSave(String fname)
	 {
		 int cntrX, cntrY, msize;

		 try
		 {
			 // Open file
			 FileWriter fos = new FileWriter(fname);
			 fos.write("<VA2>\n<MAP>\n");
			 msize = _map.getWidth();
			 for (cntrY = 0; cntrY < msize; ++cntrY)
			 {
				 for (cntrX = 0; cntrX < msize; ++cntrX)
				 {
					 fos.write(BASE64ENC.charAt(_map.getTile(cntrX, cntrY)));
				 }
				 fos.write("\n");
			 }
			 fos.write("</MAP>\n</VA2>\n");
			 fos.flush();
//			 System.out.println("File should have been written");
			 // Close file
			 fos.close();
		 }
		 catch (IOException e) {
			 System.out.println("IO Error");
		 }
        
	 }

	 public void doLoad(String fname)
	 {
		 int cntrX, cntrY, tindex, msize;
		 String s;
		
		 try
		 {
			 // Open file
			 BufferedReader fis = new BufferedReader(new FileReader(fname));
			 do {
			 s = fis.readLine();
			 } while (s.indexOf("<MAP>") < 0);
			 // read map size then set and clear (in case file problem) map 
			 msize = MAP_SIZE;
			 menu_FileNew(0);
			 for (cntrY = 0; cntrY < msize; ++cntrY)
			 {
				 s = fis.readLine();
				 for (cntrX = 0; cntrX < msize; ++cntrX)
				 {
					 tindex = s.charAt(cntrX);
					 _map.setTile(cntrX, cntrY, BASE64ENC.indexOf(tindex));
				 }
			 }

			 // Close file
			 fis.close();
			 _mapviewer.repaint();
		 }
		 catch (IOException e) {
			 System.out.println("IO Error");
		 }
        
	 }
	
	 public static void main (String args[]) {
		 // Create an instance of the mainframe
		 VA2CS mainFrame = new VA2CS();
		 // Now show it.
		 mainFrame.setVisible(true);
	 }

}
